# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.io namespace
"""

import sys as _sys

from tensorflow._api.v2.io import gfile
from tensorflow.python.ops.gen_decode_proto_ops import decode_proto_v2 as decode_proto # line: 31
from tensorflow.python.ops.gen_encode_proto_ops import encode_proto # line: 27
from tensorflow.python.ops.gen_io_ops import matching_files # line: 391
from tensorflow.python.ops.gen_io_ops import write_file # line: 2269
from tensorflow.python.ops.gen_parsing_ops import decode_compressed # line: 144
from tensorflow.python.ops.gen_parsing_ops import parse_tensor # line: 2135
from tensorflow.python.ops.gen_string_ops import decode_base64 # line: 182
from tensorflow.python.ops.gen_string_ops import encode_base64 # line: 269
from tensorflow.python.framework.graph_io import write_graph # line: 28
from tensorflow.python.lib.io.tf_record import TFRecordOptions # line: 35
from tensorflow.python.lib.io.tf_record import TFRecordWriter # line: 211
from tensorflow.python.ops.image_ops_impl import decode_and_crop_jpeg # line: 3200
from tensorflow.python.ops.image_ops_impl import decode_bmp # line: 3206
from tensorflow.python.ops.image_ops_impl import decode_gif # line: 3211
from tensorflow.python.ops.image_ops_impl import decode_image # line: 3274
from tensorflow.python.ops.image_ops_impl import decode_jpeg # line: 3216
from tensorflow.python.ops.image_ops_impl import decode_png # line: 3221
from tensorflow.python.ops.image_ops_impl import decode_webp # line: 3226
from tensorflow.python.ops.image_ops_impl import encode_jpeg # line: 3232
from tensorflow.python.ops.image_ops_impl import encode_png # line: 3244
from tensorflow.python.ops.image_ops_impl import extract_jpeg_shape # line: 3237
from tensorflow.python.ops.image_ops_impl import is_jpeg # line: 3164
from tensorflow.python.ops.io_ops import read_file # line: 97
from tensorflow.python.ops.io_ops import serialize_tensor # line: 137
from tensorflow.python.ops.parsing_config import FixedLenFeature # line: 298
from tensorflow.python.ops.parsing_config import FixedLenSequenceFeature # line: 318
from tensorflow.python.ops.parsing_config import RaggedFeature # line: 54
from tensorflow.python.ops.parsing_config import SparseFeature # line: 223
from tensorflow.python.ops.parsing_config import VarLenFeature # line: 44
from tensorflow.python.ops.parsing_ops import decode_csv_v2 as decode_csv # line: 1070
from tensorflow.python.ops.parsing_ops import decode_json_example # line: 1149
from tensorflow.python.ops.parsing_ops import decode_raw # line: 840
from tensorflow.python.ops.parsing_ops import parse_example_v2 as parse_example # line: 71
from tensorflow.python.ops.parsing_ops import parse_sequence_example # line: 449
from tensorflow.python.ops.parsing_ops import parse_single_example_v2 as parse_single_example # line: 405
from tensorflow.python.ops.parsing_ops import parse_single_sequence_example # line: 693
from tensorflow.python.ops.sparse_ops import deserialize_many_sparse # line: 2356
from tensorflow.python.ops.sparse_ops import serialize_many_sparse_v2 as serialize_many_sparse # line: 2254
from tensorflow.python.ops.sparse_ops import serialize_sparse_v2 as serialize_sparse # line: 2197
from tensorflow.python.training.input import match_filenames_once # line: 56



# Copyright 2023 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================

from typing import ClassVar, Sequence, overload

class Layout:
    @overload
    def __init__(self, layout: Layout) -> None: ...
    @overload
    def __init__(self, type: LayoutType, sharding_specs: list[str], mesh: Mesh) -> None: ...
    @overload
    def __init__(self, layout_proto) -> None: ...
    @overload
    def __init__(self, layout_str: str) -> None: ...
    @overload
    def __init__(self, mesh: Mesh, rank: int) -> None: ...
    @overload
    def __init__(self, mesh: Mesh, rank: int, batch_dim: str, axis: int) -> None: ...
    @overload
    def __init__(self, mesh: Mesh) -> None: ...
    def as_proto(self, *args, **kwargs): ...
    def global_shape_from_local_shape(self, local_shape: list[int]) -> tuple: ...
    def is_batch_parallel(self) -> bool: ...
    def is_fully_replicated(self) -> bool: ...
    def is_single_device(self) -> bool: ...
    def local_shape_from_global_shape(self, global_shape: list[int]) -> tuple: ...
    def num_shards(self, idx: int) -> int: ...
    def to_parted(self) -> Layout: ...
    def to_string(self) -> str: ...
    def __eq__(self, arg0: Layout) -> bool: ...
    @property
    def mesh(self) -> Mesh: ...
    @property
    def rank(self) -> int: ...
    @property
    def sharding_specs(self) -> list[str]: ...
    @property
    def type(self) -> LayoutType: ...

class LayoutType:
    __members__: ClassVar[dict] = ...  # read-only
    PARTED: ClassVar[LayoutType] = ...
    SINGLE_DEVICE: ClassVar[LayoutType] = ...
    STATIC: ClassVar[LayoutType] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __ne__(self, other: object) -> bool: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class Mesh:
    @overload
    def __init__(self, mesh: Mesh) -> None: ...
    @overload
    def __init__(self, arg0: str, arg1: list[str], arg2: list[int], arg3: list[int], arg4: list[str], arg5: list[int], arg6: list[str], arg7: bool) -> None: ...
    @overload
    def __init__(self, single_device: str) -> None: ...
    @overload
    def __init__(self, mesh_proto) -> None: ...
    @overload
    def __init__(self, mesh_str: str) -> None: ...
    def as_proto(self, *args, **kwargs): ...
    def contains_dim(self, dim_name: str) -> bool: ...
    def device_location(self, arg0: int) -> list[int]: ...
    def device_type(self) -> str: ...
    def dim_size(self, dim_name: str) -> int: ...
    def global_device_ids(self) -> Sequence[int]: ...
    def global_devices(self) -> list[str]: ...
    def host_mesh(self) -> Mesh: ...
    def is_remote(self) -> bool: ...
    def is_single_device(self) -> bool: ...
    def local_device_ids(self) -> Sequence[int]: ...
    def local_devices(self) -> Sequence[str]: ...
    def min_global_device_id(self) -> int: ...
    def num_local_devices(self) -> int: ...
    def shape(self) -> list[int]: ...
    def to_string(self) -> str: ...
    def use_xla_spmd(self) -> bool: ...
    def __contains__(self, dim_name: str) -> bool: ...
    def __eq__(self, arg0: Mesh) -> bool: ...
    @property
    def dim_names(self) -> list[str]: ...
    @property
    def name(self) -> str: ...
    @property
    def single_device(self) -> str: ...
    @property
    def size(self) -> int: ...

def AddMesh(arg0, arg1: str, arg2: bool) -> None: ...
def Allocate(arg0: str, arg1: bool, arg2: int) -> object: ...
def ClearTPUCoreIDs(arg0) -> None: ...
def ExperimentalClearDefaultLayout(arg0) -> None: ...
def ExperimentalClearDefaultMesh(arg0) -> None: ...
def ExperimentalSetDefaultLayout(arg0, arg1: str) -> None: ...
def ExperimentalSetDefaultMesh(arg0, arg1: str) -> None: ...
def FetchLayout(arg0: object, arg1: object, arg2) -> object: ...
def GetStats(arg0: object, arg1) -> dict[str, int]: ...
def IsDTensor(arg0: object, arg1: object, arg2) -> bool: ...
def IsSparseDTensor(arg0: object, arg1: object, arg2) -> bool: ...
def Pack(arg0: object, arg1: object, arg2: str, arg3, arg4: bool) -> object: ...
def SetIteratorElementLayouts(arg0: object, arg1: object, arg2: list[str], arg3) -> None: ...
def SetTPUCoreIDs(arg0, arg1: str, arg2: list[int]) -> None: ...
def TPUCoreIDsToLocations(arg0: object, arg1, arg2: list[int]) -> list[list[int]]: ...
def TPUCoreLocationsToIDs(arg0: object, arg1, arg2: list[list[int]]) -> list[int]: ...
def Unpack(arg0: object, arg1: object, arg2) -> object: ...



# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.dtypes namespace
"""

import sys as _sys

from tensorflow._api.v2.dtypes import experimental
from tensorflow.python.framework.dtypes import DType # line: 51
from tensorflow.python.framework.dtypes import QUANTIZED_DTYPES # line: 841
from tensorflow.python.framework.dtypes import as_dtype # line: 863
from tensorflow.python.framework.dtypes import bfloat16 # line: 420
from tensorflow.python.framework.dtypes import bool # line: 394
from tensorflow.python.framework.dtypes import complex128 # line: 382
from tensorflow.python.framework.dtypes import complex64 # line: 376
from tensorflow.python.framework.dtypes import double # line: 372
from tensorflow.python.framework.dtypes import float16 # line: 357
from tensorflow.python.framework.dtypes import float32 # line: 364
from tensorflow.python.framework.dtypes import float64 # line: 370
from tensorflow.python.framework.dtypes import half # line: 358
from tensorflow.python.framework.dtypes import int16 # line: 342
from tensorflow.python.framework.dtypes import int32 # line: 346
from tensorflow.python.framework.dtypes import int64 # line: 350
from tensorflow.python.framework.dtypes import int8 # line: 338
from tensorflow.python.framework.dtypes import qint16 # line: 402
from tensorflow.python.framework.dtypes import qint32 # line: 406
from tensorflow.python.framework.dtypes import qint8 # line: 398
from tensorflow.python.framework.dtypes import quint16 # line: 414
from tensorflow.python.framework.dtypes import quint8 # line: 410
from tensorflow.python.framework.dtypes import resource # line: 312
from tensorflow.python.framework.dtypes import string # line: 390
from tensorflow.python.framework.dtypes import uint16 # line: 326
from tensorflow.python.framework.dtypes import uint32 # line: 330
from tensorflow.python.framework.dtypes import uint64 # line: 334
from tensorflow.python.framework.dtypes import uint8 # line: 322
from tensorflow.python.framework.dtypes import variant # line: 318
from tensorflow.python.ops.math_ops import cast # line: 940
from tensorflow.python.ops.math_ops import complex # line: 695
from tensorflow.python.ops.math_ops import saturate_cast # line: 1025



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tensorflow/core/framework/kernel_def.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'tensorflow/core/framework/kernel_def.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.core.framework import attr_value_pb2 as tensorflow_dot_core_dot_framework_dot_attr__value__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*tensorflow/core/framework/kernel_def.proto\x12\ntensorflow\x1a*tensorflow/core/framework/attr_value.proto\"\xef\x01\n\tKernelDef\x12\n\n\x02op\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65vice_type\x18\x02 \x01(\t\x12\x38\n\nconstraint\x18\x03 \x03(\x0b\x32$.tensorflow.KernelDef.AttrConstraint\x12\x17\n\x0fhost_memory_arg\x18\x04 \x03(\t\x12\r\n\x05label\x18\x05 \x01(\t\x12\x10\n\x08priority\x18\x06 \x01(\x05\x1aM\n\x0e\x41ttrConstraint\x12\x0c\n\x04name\x18\x01 \x01(\t\x12-\n\x0e\x61llowed_values\x18\x02 \x01(\x0b\x32\x15.tensorflow.AttrValue\"3\n\nKernelList\x12%\n\x06kernel\x18\x01 \x03(\x0b\x32\x15.tensorflow.KernelDefB\x83\x01\n\x18org.tensorflow.frameworkB\x0fKernelDefProtosP\x01ZQgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/kernel_def_go_proto\xf8\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.framework.kernel_def_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\030org.tensorflow.frameworkB\017KernelDefProtosP\001ZQgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/kernel_def_go_proto\370\001\001'
  _globals['_KERNELDEF']._serialized_start=103
  _globals['_KERNELDEF']._serialized_end=342
  _globals['_KERNELDEF_ATTRCONSTRAINT']._serialized_start=265
  _globals['_KERNELDEF_ATTRCONSTRAINT']._serialized_end=342
  _globals['_KERNELLIST']._serialized_start=344
  _globals['_KERNELLIST']._serialized_end=395
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tensorflow/core/protobuf/config.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'tensorflow/core/protobuf/config.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.compiler.xla.tsl.protobuf import coordination_config_pb2 as xla_dot_tsl_dot_protobuf_dot_coordination__config__pb2
from tensorflow.core.framework import cost_graph_pb2 as tensorflow_dot_core_dot_framework_dot_cost__graph__pb2
from tensorflow.core.framework import graph_pb2 as tensorflow_dot_core_dot_framework_dot_graph__pb2
from tensorflow.core.framework import step_stats_pb2 as tensorflow_dot_core_dot_framework_dot_step__stats__pb2
from tensorflow.core.protobuf import cluster_pb2 as tensorflow_dot_core_dot_protobuf_dot_cluster__pb2
from tensorflow.core.protobuf import debug_pb2 as tensorflow_dot_core_dot_protobuf_dot_debug__pb2
from tensorflow.core.protobuf import rewriter_config_pb2 as tensorflow_dot_core_dot_protobuf_dot_rewriter__config__pb2
from tensorflow.core.protobuf import rpc_options_pb2 as tensorflow_dot_core_dot_protobuf_dot_rpc__options__pb2
try:
  xla_dot_tsl_dot_protobuf_dot_rpc__options__pb2 = tensorflow_dot_core_dot_protobuf_dot_rpc__options__pb2.xla_dot_tsl_dot_protobuf_dot_rpc__options__pb2
except AttributeError:
  xla_dot_tsl_dot_protobuf_dot_rpc__options__pb2 = tensorflow_dot_core_dot_protobuf_dot_rpc__options__pb2.xla.tsl.protobuf.rpc_options_pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%tensorflow/core/protobuf/config.proto\x12\ntensorflow\x1a*xla/tsl/protobuf/coordination_config.proto\x1a*tensorflow/core/framework/cost_graph.proto\x1a%tensorflow/core/framework/graph.proto\x1a*tensorflow/core/framework/step_stats.proto\x1a&tensorflow/core/protobuf/cluster.proto\x1a$tensorflow/core/protobuf/debug.proto\x1a.tensorflow/core/protobuf/rewriter_config.proto\x1a*tensorflow/core/protobuf/rpc_options.proto\"\x89\n\n\nGPUOptions\x12\'\n\x1fper_process_gpu_memory_fraction\x18\x01 \x01(\x01\x12\x14\n\x0c\x61llow_growth\x18\x04 \x01(\x08\x12\x16\n\x0e\x61llocator_type\x18\x02 \x01(\t\x12\x1f\n\x17\x64\x65\x66\x65rred_deletion_bytes\x18\x03 \x01(\x03\x12\x1b\n\x13visible_device_list\x18\x05 \x01(\t\x12\"\n\x1apolling_active_delay_usecs\x18\x06 \x01(\x05\x12$\n\x1cpolling_inactive_delay_msecs\x18\x07 \x01(\x05\x12\x1c\n\x14\x66orce_gpu_compatible\x18\x08 \x01(\x08\x12\x39\n\x0c\x65xperimental\x18\t \x01(\x0b\x32#.tensorflow.GPUOptions.Experimental\x1a\xc2\x07\n\x0c\x45xperimental\x12K\n\x0fvirtual_devices\x18\x01 \x03(\x0b\x32\x32.tensorflow.GPUOptions.Experimental.VirtualDevices\x12#\n\x1bnum_virtual_devices_per_gpu\x18\x0f \x01(\x05\x12\x1a\n\x12use_unified_memory\x18\x02 \x01(\x08\x12#\n\x1bnum_dev_to_dev_copy_streams\x18\x03 \x01(\x05\x12\x1d\n\x15\x63ollective_ring_order\x18\x04 \x01(\t\x12\x1d\n\x15timestamped_allocator\x18\x05 \x01(\x08\x12#\n\x1bkernel_tracker_max_interval\x18\x07 \x01(\x05\x12 \n\x18kernel_tracker_max_bytes\x18\x08 \x01(\x05\x12\"\n\x1akernel_tracker_max_pending\x18\t \x01(\x05\x12\'\n\x1finternal_fragmentation_fraction\x18\n \x01(\x01\x12\x1d\n\x15use_cuda_malloc_async\x18\x0b \x01(\x08\x12,\n$disallow_retry_on_allocation_failure\x18\x0c \x01(\x08\x12 \n\x18gpu_host_mem_limit_in_mb\x18\r \x01(\x02\x12$\n\x1cgpu_host_mem_disallow_growth\x18\x0e \x01(\x08\x12$\n\x1cgpu_system_memory_size_in_mb\x18\x10 \x01(\x05\x12.\n&populate_pjrt_gpu_client_creation_info\x18\x11 \x01(\x08\x12\x0f\n\x07node_id\x18\x12 \x01(\x05\x12T\n\x14stream_merge_options\x18\x13 \x01(\x0b\x32\x36.tensorflow.GPUOptions.Experimental.StreamMergeOptions\x1aS\n\x0eVirtualDevices\x12\x17\n\x0fmemory_limit_mb\x18\x01 \x03(\x02\x12\x10\n\x08priority\x18\x02 \x03(\x05\x12\x16\n\x0e\x64\x65vice_ordinal\x18\x03 \x03(\x05\x1a\x85\x01\n\x12StreamMergeOptions\x12#\n\x1bmerge_host_to_device_stream\x18\x01 \x01(\x08\x12#\n\x1bmerge_device_to_host_stream\x18\x02 \x01(\x08\x12%\n\x1dmerge_device_to_device_stream\x18\x03 \x01(\x08\"\x9d\x03\n\x10OptimizerOptions\x12+\n#do_common_subexpression_elimination\x18\x01 \x01(\x08\x12\x1b\n\x13\x64o_constant_folding\x18\x02 \x01(\x08\x12$\n\x1cmax_folded_constant_in_bytes\x18\x06 \x01(\x03\x12\x1c\n\x14\x64o_function_inlining\x18\x04 \x01(\x08\x12\x35\n\topt_level\x18\x03 \x01(\x0e\x32\".tensorflow.OptimizerOptions.Level\x12\x45\n\x10global_jit_level\x18\x05 \x01(\x0e\x32+.tensorflow.OptimizerOptions.GlobalJitLevel\x12\x16\n\x0e\x63pu_global_jit\x18\x07 \x01(\x08\" \n\x05Level\x12\x06\n\x02L1\x10\x00\x12\x0f\n\x02L0\x10\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01\"C\n\x0eGlobalJitLevel\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x10\n\x03OFF\x10\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01\x12\x08\n\x04ON_1\x10\x01\x12\x08\n\x04ON_2\x10\x02\"\xee\x02\n\x0cGraphOptions\x12\x1e\n\x16\x65nable_recv_scheduling\x18\x02 \x01(\x08\x12\x37\n\x11optimizer_options\x18\x03 \x01(\x0b\x32\x1c.tensorflow.OptimizerOptions\x12\x18\n\x10\x62uild_cost_model\x18\x04 \x01(\x03\x12\x1e\n\x16\x62uild_cost_model_after\x18\t \x01(\x03\x12\x14\n\x0cinfer_shapes\x18\x05 \x01(\x08\x12\x1a\n\x12place_pruned_graph\x18\x06 \x01(\x08\x12 \n\x18\x65nable_bfloat16_sendrecv\x18\x07 \x01(\x08\x12\x15\n\rtimeline_step\x18\x08 \x01(\x05\x12\x33\n\x0frewrite_options\x18\n \x01(\x0b\x32\x1a.tensorflow.RewriterConfigJ\x04\x08\x01\x10\x02R%skip_common_subexpression_elimination\"A\n\x15ThreadPoolOptionProto\x12\x13\n\x0bnum_threads\x18\x01 \x01(\x05\x12\x13\n\x0bglobal_name\x18\x02 \x01(\t\"0\n\x0fSessionMetadata\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\x03\"\xb4\x11\n\x0b\x43onfigProto\x12>\n\x0c\x64\x65vice_count\x18\x01 \x03(\x0b\x32(.tensorflow.ConfigProto.DeviceCountEntry\x12$\n\x1cintra_op_parallelism_threads\x18\x02 \x01(\x05\x12$\n\x1cinter_op_parallelism_threads\x18\x05 \x01(\x05\x12\x1f\n\x17use_per_session_threads\x18\t \x01(\x08\x12G\n\x1csession_inter_op_thread_pool\x18\x0c \x03(\x0b\x32!.tensorflow.ThreadPoolOptionProto\x12\x18\n\x10placement_period\x18\x03 \x01(\x05\x12\x16\n\x0e\x64\x65vice_filters\x18\x04 \x03(\t\x12+\n\x0bgpu_options\x18\x06 \x01(\x0b\x32\x16.tensorflow.GPUOptions\x12\x38\n\x18pluggable_device_options\x18\x12 \x01(\x0b\x32\x16.tensorflow.GPUOptions\x12\x1c\n\x14\x61llow_soft_placement\x18\x07 \x01(\x08\x12\x1c\n\x14log_device_placement\x18\x08 \x01(\x08\x12/\n\rgraph_options\x18\n \x01(\x0b\x32\x18.tensorflow.GraphOptions\x12\x1f\n\x17operation_timeout_in_ms\x18\x0b \x01(\x03\x12+\n\x0brpc_options\x18\r \x01(\x0b\x32\x16.tensorflow.RPCOptions\x12+\n\x0b\x63luster_def\x18\x0e \x01(\x0b\x32\x16.tensorflow.ClusterDef\x12\x1d\n\x15isolate_session_state\x18\x0f \x01(\x08\x12(\n share_cluster_devices_in_session\x18\x11 \x01(\x08\x12:\n\x0c\x65xperimental\x18\x10 \x01(\x0b\x32$.tensorflow.ConfigProto.Experimental\x1a\x32\n\x10\x44\x65viceCountEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a\xf4\n\n\x0c\x45xperimental\x12\x1f\n\x17\x63ollective_group_leader\x18\x01 \x01(\t\x12\x15\n\rexecutor_type\x18\x03 \x01(\t\x12\x1a\n\x12recv_buf_max_chunk\x18\x04 \x01(\x05\x12\x19\n\x11use_numa_affinity\x18\x05 \x01(\x08\x12\x35\n-collective_deterministic_sequential_execution\x18\x06 \x01(\x08\x12\x17\n\x0f\x63ollective_nccl\x18\x07 \x01(\x08\x12\x36\n.share_session_state_in_clusterspec_propagation\x18\x08 \x01(\x08\x12\x1f\n\x17\x64isable_thread_spinning\x18\t \x01(\x08\x12(\n share_cluster_devices_in_session\x18\n \x01(\x08\x12\x35\n\x10session_metadata\x18\x0b \x01(\x0b\x32\x1b.tensorflow.SessionMetadata\x12!\n\x19optimize_for_static_graph\x18\x0c \x01(\x08\x12\x1a\n\x12\x65nable_mlir_bridge\x18\r \x01(\x08\x12S\n\x13mlir_bridge_rollout\x18\x11 \x01(\x0e\x32\x36.tensorflow.ConfigProto.Experimental.MlirBridgeRollout\x12&\n\x1e\x65nable_mlir_graph_optimization\x18\x10 \x01(\x08\x12\'\n\x1f\x64isable_output_partition_graphs\x18\x0e \x01(\x08\x12#\n\x1bxla_fusion_autotuner_thresh\x18\x0f \x01(\x03\x12\x10\n\x08use_tfrt\x18\x12 \x01(\x08\x12\x19\n\x11\x65nable_multi_host\x18\x1b \x01(\x08\x12\x15\n\rtfrt_use_ifrt\x18  \x01(\x08\x12\x1b\n\x13\x62\x61\x63kend_server_port\x18\x1c \x01(\x05\x12\x12\n\ntarget_tpu\x18\x1d \x01(\x08\x12\x12\n\ntarget_gpu\x18\x1e \x01(\x08\x12\x1e\n\x16stream_merge_threshold\x18\x1f \x01(\x05\x12\'\n\x1f\x64isable_functional_ops_lowering\x18\x15 \x01(\x08\x12\'\n\x1fxla_prefer_single_graph_cluster\x18\x16 \x01(\x08\x12\x42\n\x13\x63oordination_config\x18\x17 \x01(\x0b\x32%.tensorflow.CoordinationServiceConfig\x12)\n!disable_optimize_for_static_graph\x18\x18 \x01(\x08\x12\x30\n(disable_eager_executor_streaming_enqueue\x18\x1a \x01(\x08\x12)\n!finalize_function_library_runtime\x18! \x01(\x08\x12!\n\x19\x66inalize_resource_manager\x18\" \x01(\x08\"\xde\x01\n\x11MlirBridgeRollout\x12#\n\x1fMLIR_BRIDGE_ROLLOUT_UNSPECIFIED\x10\x00\x12\x1f\n\x1bMLIR_BRIDGE_ROLLOUT_ENABLED\x10\x01\x12 \n\x1cMLIR_BRIDGE_ROLLOUT_DISABLED\x10\x02\"\x04\x08\x03\x10\x03\"\x04\x08\x04\x10\x04*%MLIR_BRIDGE_ROLLOUT_SAFE_MODE_ENABLED*.MLIR_BRIDGE_ROLLOUT_SAFE_MODE_FALLBACK_ENABLEDJ\x04\x08\x02\x10\x03J\x04\x08\x13\x10\x14J\x04\x08\x14\x10\x15J\x04\x08\x19\x10\x1a\"\xe1\x04\n\nRunOptions\x12\x36\n\x0btrace_level\x18\x01 \x01(\x0e\x32!.tensorflow.RunOptions.TraceLevel\x12\x15\n\rtimeout_in_ms\x18\x02 \x01(\x03\x12\x1c\n\x14inter_op_thread_pool\x18\x03 \x01(\x05\x12\x1f\n\x17output_partition_graphs\x18\x05 \x01(\x08\x12/\n\rdebug_options\x18\x06 \x01(\x0b\x32\x18.tensorflow.DebugOptions\x12*\n\"report_tensor_allocations_upon_oom\x18\x07 \x01(\x08\x12\x39\n\x0c\x65xperimental\x18\x08 \x01(\x0b\x32#.tensorflow.RunOptions.Experimental\x1a\xd2\x01\n\x0c\x45xperimental\x12\x1c\n\x14\x63ollective_graph_key\x18\x01 \x01(\x03\x12\x1c\n\x14use_run_handler_pool\x18\x02 \x01(\x08\x12[\n\x18run_handler_pool_options\x18\x03 \x01(\x0b\x32\x39.tensorflow.RunOptions.Experimental.RunHandlerPoolOptions\x1a)\n\x15RunHandlerPoolOptions\x12\x10\n\x08priority\x18\x01 \x01(\x03\"R\n\nTraceLevel\x12\x0c\n\x08NO_TRACE\x10\x00\x12\x12\n\x0eSOFTWARE_TRACE\x10\x01\x12\x12\n\x0eHARDWARE_TRACE\x10\x02\x12\x0e\n\nFULL_TRACE\x10\x03J\x04\x08\x04\x10\x05\"\xbe\x03\n\x0bRunMetadata\x12)\n\nstep_stats\x18\x01 \x01(\x0b\x32\x15.tensorflow.StepStats\x12,\n\ncost_graph\x18\x02 \x01(\x0b\x32\x18.tensorflow.CostGraphDef\x12.\n\x10partition_graphs\x18\x03 \x03(\x0b\x32\x14.tensorflow.GraphDef\x12?\n\x0f\x66unction_graphs\x18\x04 \x03(\x0b\x32&.tensorflow.RunMetadata.FunctionGraphs\x12\x35\n\x10session_metadata\x18\x05 \x01(\x0b\x32\x1b.tensorflow.SessionMetadata\x1a\xad\x01\n\x0e\x46unctionGraphs\x12.\n\x10partition_graphs\x18\x01 \x03(\x0b\x32\x14.tensorflow.GraphDef\x12\x34\n\x16pre_optimization_graph\x18\x02 \x01(\x0b\x32\x14.tensorflow.GraphDef\x12\x35\n\x17post_optimization_graph\x18\x03 \x01(\x0b\x32\x14.tensorflow.GraphDef\":\n\x10TensorConnection\x12\x13\n\x0b\x66rom_tensor\x18\x01 \x01(\t\x12\x11\n\tto_tensor\x18\x02 \x01(\t\"\xb0\x03\n\x0f\x43\x61llableOptions\x12\x0c\n\x04\x66\x65\x65\x64\x18\x01 \x03(\t\x12\r\n\x05\x66\x65tch\x18\x02 \x03(\t\x12\x0e\n\x06target\x18\x03 \x03(\t\x12+\n\x0brun_options\x18\x04 \x01(\x0b\x32\x16.tensorflow.RunOptions\x12\x37\n\x11tensor_connection\x18\x05 \x03(\x0b\x32\x1c.tensorflow.TensorConnection\x12\x42\n\x0c\x66\x65\x65\x64_devices\x18\x06 \x03(\x0b\x32,.tensorflow.CallableOptions.FeedDevicesEntry\x12\x44\n\rfetch_devices\x18\x07 \x03(\x0b\x32-.tensorflow.CallableOptions.FetchDevicesEntry\x12\x17\n\x0f\x66\x65tch_skip_sync\x18\x08 \x01(\x08\x1a\x32\n\x10\x46\x65\x65\x64\x44\x65vicesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x33\n\x11\x46\x65tchDevicesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x9d\x01\n\x0f\x42\x61tchingOptions\x12\x19\n\x11num_batch_threads\x18\x01 \x01(\x05\x12\x16\n\x0emax_batch_size\x18\x02 \x01(\x05\x12\x1c\n\x14\x62\x61tch_timeout_micros\x18\x03 \x01(\x05\x12\x1b\n\x13\x61llowed_batch_sizes\x18\x04 \x03(\x05\x12\x1c\n\x14max_enqueued_batches\x18\x05 \x01(\x05\x42\x84\x01\n\x18org.tensorflow.frameworkB\x0c\x43onfigProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.protobuf.config_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\030org.tensorflow.frameworkB\014ConfigProtosP\001ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\370\001\001'
  _globals['_CONFIGPROTO_DEVICECOUNTENTRY']._loaded_options = None
  _globals['_CONFIGPROTO_DEVICECOUNTENTRY']._serialized_options = b'8\001'
  _globals['_CALLABLEOPTIONS_FEEDDEVICESENTRY']._loaded_options = None
  _globals['_CALLABLEOPTIONS_FEEDDEVICESENTRY']._serialized_options = b'8\001'
  _globals['_CALLABLEOPTIONS_FETCHDEVICESENTRY']._loaded_options = None
  _globals['_CALLABLEOPTIONS_FETCHDEVICESENTRY']._serialized_options = b'8\001'
  _globals['_GPUOPTIONS']._serialized_start=395
  _globals['_GPUOPTIONS']._serialized_end=1684
  _globals['_GPUOPTIONS_EXPERIMENTAL']._serialized_start=722
  _globals['_GPUOPTIONS_EXPERIMENTAL']._serialized_end=1684
  _globals['_GPUOPTIONS_EXPERIMENTAL_VIRTUALDEVICES']._serialized_start=1465
  _globals['_GPUOPTIONS_EXPERIMENTAL_VIRTUALDEVICES']._serialized_end=1548
  _globals['_GPUOPTIONS_EXPERIMENTAL_STREAMMERGEOPTIONS']._serialized_start=1551
  _globals['_GPUOPTIONS_EXPERIMENTAL_STREAMMERGEOPTIONS']._serialized_end=1684
  _globals['_OPTIMIZEROPTIONS']._serialized_start=1687
  _globals['_OPTIMIZEROPTIONS']._serialized_end=2100
  _globals['_OPTIMIZEROPTIONS_LEVEL']._serialized_start=1999
  _globals['_OPTIMIZEROPTIONS_LEVEL']._serialized_end=2031
  _globals['_OPTIMIZEROPTIONS_GLOBALJITLEVEL']._serialized_start=2033
  _globals['_OPTIMIZEROPTIONS_GLOBALJITLEVEL']._serialized_end=2100
  _globals['_GRAPHOPTIONS']._serialized_start=2103
  _globals['_GRAPHOPTIONS']._serialized_end=2469
  _globals['_THREADPOOLOPTIONPROTO']._serialized_start=2471
  _globals['_THREADPOOLOPTIONPROTO']._serialized_end=2536
  _globals['_SESSIONMETADATA']._serialized_start=2538
  _globals['_SESSIONMETADATA']._serialized_end=2586
  _globals['_CONFIGPROTO']._serialized_start=2589
  _globals['_CONFIGPROTO']._serialized_end=4817
  _globals['_CONFIGPROTO_DEVICECOUNTENTRY']._serialized_start=3368
  _globals['_CONFIGPROTO_DEVICECOUNTENTRY']._serialized_end=3418
  _globals['_CONFIGPROTO_EXPERIMENTAL']._serialized_start=3421
  _globals['_CONFIGPROTO_EXPERIMENTAL']._serialized_end=4817
  _globals['_CONFIGPROTO_EXPERIMENTAL_MLIRBRIDGEROLLOUT']._serialized_start=4571
  _globals['_CONFIGPROTO_EXPERIMENTAL_MLIRBRIDGEROLLOUT']._serialized_end=4793
  _globals['_RUNOPTIONS']._serialized_start=4820
  _globals['_RUNOPTIONS']._serialized_end=5429
  _globals['_RUNOPTIONS_EXPERIMENTAL']._serialized_start=5129
  _globals['_RUNOPTIONS_EXPERIMENTAL']._serialized_end=5339
  _globals['_RUNOPTIONS_EXPERIMENTAL_RUNHANDLERPOOLOPTIONS']._serialized_start=5298
  _globals['_RUNOPTIONS_EXPERIMENTAL_RUNHANDLERPOOLOPTIONS']._serialized_end=5339
  _globals['_RUNOPTIONS_TRACELEVEL']._serialized_start=5341
  _globals['_RUNOPTIONS_TRACELEVEL']._serialized_end=5423
  _globals['_RUNMETADATA']._serialized_start=5432
  _globals['_RUNMETADATA']._serialized_end=5878
  _globals['_RUNMETADATA_FUNCTIONGRAPHS']._serialized_start=5705
  _globals['_RUNMETADATA_FUNCTIONGRAPHS']._serialized_end=5878
  _globals['_TENSORCONNECTION']._serialized_start=5880
  _globals['_TENSORCONNECTION']._serialized_end=5938
  _globals['_CALLABLEOPTIONS']._serialized_start=5941
  _globals['_CALLABLEOPTIONS']._serialized_end=6373
  _globals['_CALLABLEOPTIONS_FEEDDEVICESENTRY']._serialized_start=6270
  _globals['_CALLABLEOPTIONS_FEEDDEVICESENTRY']._serialized_end=6320
  _globals['_CALLABLEOPTIONS_FETCHDEVICESENTRY']._serialized_start=6322
  _globals['_CALLABLEOPTIONS_FETCHDEVICESENTRY']._serialized_end=6373
  _globals['_BATCHINGOPTIONS']._serialized_start=6376
  _globals['_BATCHINGOPTIONS']._serialized_end=6533
# @@protoc_insertion_point(module_scope)



# Copyright 2023 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================

from typing import overload

def ExperimentalConvertSavedModelToMlir(arg0: str, arg1: str, arg2: bool) -> str: ...
def ExperimentalConvertSavedModelV1ToMlir(arg0: str, arg1: str, arg2: str, arg3: bool, arg4: bool, arg5: bool, arg6: bool) -> str: ...
def ExperimentalConvertSavedModelV1ToMlirLite(arg0: str, arg1: str, arg2: str, arg3: bool, arg4: bool) -> str: ...
def ExperimentalRunPassPipeline(arg0: str, arg1: str, arg2: bool) -> str: ...
def ExperimentalWriteBytecode(arg0: str, arg1: str) -> None: ...
def ImportFunction(arg0: object, arg1: str, arg2: str, arg3: bool) -> str: ...
@overload
def ImportGraphDef(arg0: str, arg1: str, arg2: bool) -> str: ...
@overload
def ImportGraphDef(arg0: str, arg1: str, arg2: bool, arg3: str, arg4: str, arg5: str, arg6: str) -> str: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tensorflow/core/protobuf/tpu/compilation_result.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'tensorflow/core/protobuf/tpu/compilation_result.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.compiler.xla.service import hlo_pb2 as xla_dot_service_dot_hlo__pb2
from tensorflow.core.protobuf import error_codes_pb2 as tensorflow_dot_core_dot_protobuf_dot_error__codes__pb2
try:
  xla_dot_tsl_dot_protobuf_dot_error__codes__pb2 = tensorflow_dot_core_dot_protobuf_dot_error__codes__pb2.xla_dot_tsl_dot_protobuf_dot_error__codes__pb2
except AttributeError:
  xla_dot_tsl_dot_protobuf_dot_error__codes__pb2 = tensorflow_dot_core_dot_protobuf_dot_error__codes__pb2.xla.tsl.protobuf.error_codes_pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5tensorflow/core/protobuf/tpu/compilation_result.proto\x12\x0etensorflow.tpu\x1a\x15xla/service/hlo.proto\x1a*tensorflow/core/protobuf/error_codes.proto\"\xf9\x01\n\x16\x43ompilationResultProto\x12+\n\x0bstatus_code\x18\x01 \x01(\x0e\x32\x16.tensorflow.error.Code\x12\x1c\n\x14status_error_message\x18\x02 \x01(\t\x12!\n\nhlo_protos\x18\x03 \x03(\x0b\x32\r.xla.HloProto\x12\x44\n\nerror_code\x18\x04 \x01(\x0e\x32\x30.tensorflow.tpu.CompilationResultProto.ErrorCode\"+\n\tErrorCode\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x11\n\rOUT_OF_MEMORY\x10\x01\x42\x03\xf8\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.protobuf.tpu.compilation_result_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\370\001\001'
  _globals['_COMPILATIONRESULTPROTO']._serialized_start=141
  _globals['_COMPILATIONRESULTPROTO']._serialized_end=390
  _globals['_COMPILATIONRESULTPROTO_ERRORCODE']._serialized_start=347
  _globals['_COMPILATIONRESULTPROTO_ERRORCODE']._serialized_end=390
# @@protoc_insertion_point(module_scope)



from sys import modules
from types import ModuleType


def __update_globals(new_import_path, pywrap_m):
  all_names = pywrap_m.__all__ if hasattr(pywrap_m, '__all__') else dir(
      pywrap_m)
  modules[new_import_path] = pywrap_m
  for name in all_names:
    sub_pywrap = getattr(pywrap_m, name)
    if isinstance(sub_pywrap, ModuleType):
      sub_name = sub_pywrap.__name__[len(pywrap_m.__name__):]
      __update_globals(new_import_path + sub_name, sub_pywrap)


def __try_import():
  imports_paths = ["litert.python.format_converter_wrapper_pybind11", "third_party.tensorflow.lite.python.format_converter_wrapper_pybind11", "tensorflow.format_converter_wrapper_pybind11", "tensorflow.python.format_converter_wrapper_pybind11"] # template_val
  exceptions = []
  last_exception = None
  for import_path in imports_paths:
    try:
      pywrap_m = __import__(import_path, fromlist=["*"])
      __update_globals(__name__, pywrap_m)
      return
    except ImportError as e:
      exceptions.append(str(e))
      last_exception = e
      pass

  raise RuntimeError(f"""
Could not import original test/binary location, import paths tried: {imports_paths}. 
Previous exceptions: {exceptions}""", last_exception)


__try_import()



# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.train namespace
"""

import sys as _sys

from tensorflow._api.v2.train import experimental
from tensorflow.python.checkpoint.checkpoint import Checkpoint # line: 2060
from tensorflow.python.checkpoint.checkpoint_management import CheckpointManager # line: 518
from tensorflow.python.checkpoint.checkpoint_management import get_checkpoint_state # line: 250
from tensorflow.python.checkpoint.checkpoint_management import latest_checkpoint # line: 328
from tensorflow.python.checkpoint.checkpoint_options import CheckpointOptions # line: 25
from tensorflow.python.checkpoint.checkpoint_view import CheckpointView # line: 28
from tensorflow.python.checkpoint.trackable_view import TrackableView # line: 25
from tensorflow.python.training.checkpoint_utils import checkpoints_iterator # line: 181
from tensorflow.python.training.checkpoint_utils import list_variables # line: 117
from tensorflow.python.training.checkpoint_utils import load_checkpoint # line: 46
from tensorflow.python.training.checkpoint_utils import load_variable # line: 83
from tensorflow.python.training.coordinator import Coordinator # line: 27
from tensorflow.python.training.moving_averages import ExponentialMovingAverage # line: 283
from tensorflow.python.training.server_lib import ClusterSpec # line: 242
from tensorflow.python.training.training import BytesList # line: 131
from tensorflow.python.training.training import ClusterDef # line: 132
from tensorflow.python.training.training import Example # line: 133
from tensorflow.python.training.training import Feature # line: 134
from tensorflow.python.training.training import FeatureList # line: 136
from tensorflow.python.training.training import FeatureLists # line: 137
from tensorflow.python.training.training import Features # line: 135
from tensorflow.python.training.training import FloatList # line: 138
from tensorflow.python.training.training import Int64List # line: 139
from tensorflow.python.training.training import JobDef # line: 140
from tensorflow.python.training.training import SequenceExample # line: 142
from tensorflow.python.training.training import ServerDef # line: 143



# Copyright 2021 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""The execution context for ClusterCoordinator."""

import contextlib
import threading

from tensorflow.core.framework import attr_value_pb2
from tensorflow.python.distribute.coordinator import remote_value
from tensorflow.python.framework import dtypes
from tensorflow.python.framework import ops
from tensorflow.python.framework import tensor
from tensorflow.python.util import compat
from tensorflow.python.util.tf_export import tf_export

_dispatch_context = threading.local()


def get_current_dispatch_context():
  try:
    return _dispatch_context.current
  except AttributeError:
    return None


@contextlib.contextmanager
def with_dispatch_context(worker_obj):
  previous_context = getattr(_dispatch_context, "current", None)
  _dispatch_context.current = DispatchContext(worker_obj)
  yield
  _dispatch_context.current = previous_context


class DispatchContext(object):
  """Context entered when executing a closure on a given worker."""

  def __init__(self, worker_obj):
    self._worker = worker_obj
    self._worker_index = worker_obj.worker_index

  @property
  def worker(self):
    return self._worker

  @property
  def worker_index(self):
    return self._worker_index

  def maybe_get_remote_value(self, ret):
    return maybe_get_remote_value(ret)


def maybe_get_remote_value(val):
  """Gets the value of `val` if it is a `RemoteValue`."""
  if isinstance(val, remote_value.RemoteValue):
    error = val._get_error()  # pylint: disable=protected-access
    if error:
      raise AssertionError(
          "RemoteValue doesn't have a value because it has error %r:%s" %
          (error, error))
    elif val._status is not remote_value.RemoteValueStatus.READY:  # pylint: disable=protected-access
      raise AssertionError("The input RemoteValue has not been executed.")
    else:
      return val._get_values()  # pylint: disable=protected-access
  else:
    return val


@tf_export("distribute.coordinator.experimental_get_current_worker_index",
           v1=[])
def get_current_worker_index():
  """Returns the current worker index, when called within a worker closure.

  Some parameter server training workloads may require the worker to know its
  index, for example for data sharding for reduced-variance training.

  This method may be used within a `tf.function` that is executed on a worker.
  That is, either a `dataset_fn` that runs via
  `ClusterCoordinator.create_per_worker_dataset`, or any other function
  scheduled via `ClusterCoordinator.schedule`.

  Example (sharding data by worker):

  ```python
  strategy = tf.distribute.ParameterServerStrategy(
      cluster_resolver=...)
  coordinator = (
      tf.distribute.coordinator.ClusterCoordinator(strategy))

  def dataset_fn(context):
    dataset = tf.data.Dataset.range(10)
    worker_index = (
        tf.distribute.coordinator.experimental_get_current_worker_index()
    )
    dataset = dataset.shard(
        num_shards=num_workers,
        index=worker_index,
    )
    return dataset

  @tf.function
  def per_worker_dataset_fn():
    return strategy.distribute_datasets_from_function(dataset_fn)

  per_worker_dataset = coordinator.create_per_worker_dataset(
      per_worker_dataset_fn)
  ```

  Raises:
    RuntimeError: if called from outside a `tf.function` or outside of a remote
      closure execution context (that is, on a non-worker machine).
  """

  msg = ("Cannot retrieve the worker index. `get_worker_idx_and_num_workers` "
         "should be called from within a tf.function being executed on a "
         "worker. This method should only be called from either a dataset_fn "
         "that is passed into `ClusterCoordinator.create_per_worker_dataset`, "
         "or a tf.function that is passed into `ClusterCoordinator.schedule`.")
  if not ops.inside_function():
    raise RuntimeError(msg)

  def call_time_worker_index():
    dispatch_context = get_current_dispatch_context()
    if not dispatch_context:
      raise RuntimeError(msg)
    return dispatch_context.worker_index

  worker_index = ops.get_default_graph().capture_call_time_value(
      call_time_worker_index, tensor.TensorSpec([], dtype=dtypes.int64))
  worker_index.op._set_attr(  # pylint: disable=protected-access
      "_user_specified_name",
      attr_value_pb2.AttrValue(s=compat.as_bytes("worker_index")))
  return worker_index



# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.distribute.experimental.coordinator namespace
"""

import sys as _sys

from tensorflow.python.distribute.coordinator.cluster_coordinator import ClusterCoordinator # line: 1397
from tensorflow.python.distribute.coordinator.remote_value import RemoteValue # line: 53
from tensorflow.python.distribute.coordinator.values import PerWorkerValues # line: 152



# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.__internal__.nest namespace
"""

import sys as _sys

from tensorflow.python.util.nest import flatten_up_to # line: 764
from tensorflow.python.util.nest import get_traverse_shallow_structure # line: 1115
from tensorflow.python.util.nest import is_attrs # line: 138
from tensorflow.python.util.nest import is_mapping # line: 131
from tensorflow.python.util.nest import list_to_tuple # line: 1288
from tensorflow.python.util.nest import map_structure_up_to # line: 960
from tensorflow.python.util.nest import _sequence_like as sequence_like # line: 144
from tensorflow.python.util.nest import yield_flat_paths # line: 1188



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tensorflow/core/protobuf/rewriter_config.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'tensorflow/core/protobuf/rewriter_config.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.core.framework import attr_value_pb2 as tensorflow_dot_core_dot_framework_dot_attr__value__pb2
from tensorflow.core.protobuf import verifier_config_pb2 as tensorflow_dot_core_dot_protobuf_dot_verifier__config__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.tensorflow/core/protobuf/rewriter_config.proto\x12\ntensorflow\x1a*tensorflow/core/framework/attr_value.proto\x1a.tensorflow/core/protobuf/verifier_config.proto\";\n\x13\x41utoParallelOptions\x12\x0e\n\x06\x65nable\x18\x01 \x01(\x08\x12\x14\n\x0cnum_replicas\x18\x02 \x01(\x05\"+\n\x16ScopedAllocatorOptions\x12\x11\n\tenable_op\x18\x01 \x03(\t\"\x95\x16\n\x0eRewriterConfig\x12\x43\n\x15\x63pu_layout_conversion\x18\x32 \x01(\x0e\x32$.tensorflow.RewriterConfig.CpuLayout\x12;\n\x10layout_optimizer\x18\x01 \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12;\n\x10\x63onstant_folding\x18\x03 \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12=\n\x12shape_optimization\x18\r \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12\x34\n\tremapping\x18\x0e \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12\x46\n\x1b\x63ommon_subgraph_elimination\x18\x18 \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12\x42\n\x17\x61rithmetic_optimization\x18\x07 \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12\x42\n\x17\x64\x65pendency_optimization\x18\x08 \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12<\n\x11loop_optimization\x18\t \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12@\n\x15\x66unction_optimization\x18\n \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12\x39\n\x0e\x64\x65\x62ug_stripper\x18\x0b \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12\x1d\n\x15\x64isable_model_pruning\x18\x02 \x01(\x08\x12H\n\x1dscoped_allocator_optimization\x18\x0f \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12\x43\n\x18pin_to_host_optimization\x18\x12 \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12\x42\n\x17implementation_selector\x18\x16 \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12?\n\x14\x61uto_mixed_precision\x18\x17 \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12\x43\n\x18\x61uto_mixed_precision_mkl\x18\x19 \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12O\n$auto_mixed_precision_onednn_bfloat16\x18\x1f \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12\x43\n\x18\x61uto_mixed_precision_cpu\x18\x1d \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12\x1e\n\x16\x64isable_meta_optimizer\x18\x13 \x01(\x08\x12\x1d\n\x15\x64isable_tfg_optimizer\x18  \x01(\x08\x12@\n\x15use_plugin_optimizers\x18\x1c \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12O\n$experimental_conditional_code_motion\x18\x1e \x01(\x0e\x32!.tensorflow.RewriterConfig.Toggle\x12O\n\x19meta_optimizer_iterations\x18\x0c \x01(\x0e\x32,.tensorflow.RewriterConfig.NumIterationsType\x12\x17\n\x0fmin_graph_nodes\x18\x11 \x01(\x05\x12;\n3experimental_disable_compressed_tensor_optimization\x18\x1a \x01(\x08\x12;\n3experimental_disable_folding_quantization_emulation\x18\x1b \x01(\x08\x12\x42\n\x13memory_optimization\x18\x04 \x01(\x0e\x32%.tensorflow.RewriterConfig.MemOptType\x12/\n\'memory_optimizer_target_node_name_scope\x18\x06 \x01(\t\x12!\n\x19meta_optimizer_timeout_ms\x18\x14 \x01(\x03\x12\x36\n\rauto_parallel\x18\x05 \x01(\x0b\x32\x1f.tensorflow.AutoParallelOptions\x12 \n\x18\x66\x61il_on_optimizer_errors\x18\x15 \x01(\x08\x12\x41\n\x15scoped_allocator_opts\x18\x10 \x01(\x0b\x32\".tensorflow.ScopedAllocatorOptions\x12\x12\n\noptimizers\x18\x64 \x03(\t\x12K\n\x11\x63ustom_optimizers\x18\xc8\x01 \x03(\x0b\x32/.tensorflow.RewriterConfig.CustomGraphOptimizer\x12\x44\n\x1finter_optimizer_verifier_config\x18\xac\x02 \x01(\x0b\x32\x1a.tensorflow.VerifierConfig\x12\x46\n!post_optimization_verifier_config\x18\xad\x02 \x01(\x0b\x32\x1a.tensorflow.VerifierConfig\x1a\xca\x01\n\x14\x43ustomGraphOptimizer\x12\x0c\n\x04name\x18\x01 \x01(\t\x12X\n\rparameter_map\x18\x02 \x03(\x0b\x32\x41.tensorflow.RewriterConfig.CustomGraphOptimizer.ParameterMapEntry\x1aJ\n\x11ParameterMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12$\n\x05value\x18\x02 \x01(\x0b\x32\x15.tensorflow.AttrValue:\x02\x38\x01\"d\n\x06Toggle\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x06\n\x02ON\x10\x01\x12\x07\n\x03OFF\x10\x02\x12\x0e\n\nAGGRESSIVE\x10\x03\x12\x15\n\x11\x45XPERIMENTAL_MLIR\x10\x04\x12\x15\n\x11\x45XPERIMENTAL_BOTH\x10\x05\"I\n\tCpuLayout\x12\x18\n\x14NO_CONVERSION_ON_CPU\x10\x00\x12\x10\n\x0cNCHW_TO_NHWC\x10\x01\x12\x10\n\x0cNHWC_TO_NCHW\x10\x02\"<\n\x11NumIterationsType\x12\x15\n\x11\x44\x45\x46\x41ULT_NUM_ITERS\x10\x00\x12\x07\n\x03ONE\x10\x01\x12\x07\n\x03TWO\x10\x02\"\x9f\x01\n\nMemOptType\x12\x13\n\x0f\x44\x45\x46\x41ULT_MEM_OPT\x10\x00\x12\x0e\n\nNO_MEM_OPT\x10\x01\x12\n\n\x06MANUAL\x10\x02\x12\x17\n\x13SWAPPING_HEURISTICS\x10\x04\x12\x1c\n\x18RECOMPUTATION_HEURISTICS\x10\x05\x12\x19\n\x15SCHEDULING_HEURISTICS\x10\x06\x12\x0e\n\nHEURISTICS\x10\x03\x42\x8c\x01\n\x18org.tensorflow.frameworkB\x14RewriterConfigProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.protobuf.rewriter_config_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\030org.tensorflow.frameworkB\024RewriterConfigProtosP\001ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\370\001\001'
  _globals['_REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER_PARAMETERMAPENTRY']._loaded_options = None
  _globals['_REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER_PARAMETERMAPENTRY']._serialized_options = b'8\001'
  _globals['_AUTOPARALLELOPTIONS']._serialized_start=154
  _globals['_AUTOPARALLELOPTIONS']._serialized_end=213
  _globals['_SCOPEDALLOCATOROPTIONS']._serialized_start=215
  _globals['_SCOPEDALLOCATOROPTIONS']._serialized_end=258
  _globals['_REWRITERCONFIG']._serialized_start=261
  _globals['_REWRITERCONFIG']._serialized_end=3098
  _globals['_REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER']._serialized_start=2495
  _globals['_REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER']._serialized_end=2697
  _globals['_REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER_PARAMETERMAPENTRY']._serialized_start=2623
  _globals['_REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER_PARAMETERMAPENTRY']._serialized_end=2697
  _globals['_REWRITERCONFIG_TOGGLE']._serialized_start=2699
  _globals['_REWRITERCONFIG_TOGGLE']._serialized_end=2799
  _globals['_REWRITERCONFIG_CPULAYOUT']._serialized_start=2801
  _globals['_REWRITERCONFIG_CPULAYOUT']._serialized_end=2874
  _globals['_REWRITERCONFIG_NUMITERATIONSTYPE']._serialized_start=2876
  _globals['_REWRITERCONFIG_NUMITERATIONSTYPE']._serialized_end=2936
  _globals['_REWRITERCONFIG_MEMOPTTYPE']._serialized_start=2939
  _globals['_REWRITERCONFIG_MEMOPTTYPE']._serialized_end=3098
# @@protoc_insertion_point(module_scope)



# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator2/generator/generator.py script.
"""Public API for tf._api.v2.mlir.experimental namespace
"""

import sys as _sys

from tensorflow.python.compiler.mlir.mlir import convert_function # line: 48
from tensorflow.python.compiler.mlir.mlir import convert_graph_def # line: 21
from tensorflow.python.compiler.mlir.mlir import convert_saved_model # line: 95
from tensorflow.python.compiler.mlir.mlir import convert_saved_model_v1 # line: 115
from tensorflow.python.compiler.mlir.mlir import run_pass_pipeline # line: 152
from tensorflow.python.compiler.mlir.mlir import tflite_to_tosa_bytecode # line: 181
from tensorflow.python.compiler.mlir.mlir import experimental_write_bytecode as write_bytecode # line: 170
